package tofly.opc_gateway;

import lombok.AllArgsConstructor;
import lombok.Data;
import org.jinterop.dcom.common.JIException;
import org.jinterop.dcom.core.JIVariant;
import org.openscada.opc.lib.da.DataCallback;
import org.openscada.opc.lib.da.Item;
import org.openscada.opc.lib.da.ItemState;
import tofly.opc_gateway.log.Log;
import tofly.opc_gateway.log.LogFactory;
import tofly.opc_gateway.transmit.TransmitToMqtt;
import tofly.opc_gateway.util.DateTimeUtil;
import tofly.opc_gateway.util.JsonItemConst;
import tofly.opc_gateway.util.JsonUtil;

import java.text.SimpleDateFormat;
import java.util.LinkedHashMap;
import java.util.Map;

@Data
@AllArgsConstructor
public class ServerDataCallback implements DataCallback {

    private static final Log log = LogFactory.getLog(ServerDataCallback.class);

    private String deviceSn;
    private Map<String, String> itemMappings;

    @Override
    public void changed(Item item, ItemState state) {
        try {
            int type = state.getValue().getType();
            Object value = null;
            Map<String, Object> data=new LinkedHashMap<>();
            switch (type) {
                case JIVariant.VT_I2: // 整形占2个字节，即 short 类型
                    value = state.getValue().getObjectAsShort();
                    break;
                case JIVariant.VT_I4: // 整形占4个字节，即 int 类型，KEPServerEX 上的类型是 Long
                    value = state.getValue().getObjectAsInt();
                    break;
                case JIVariant.VT_I8: // 整形占8个字节，即 long 类型，KEPServerEX 上的类型是 LLong
                    value = state.getValue().getObjectAsLong();
                    break;
                case JIVariant.VT_R4: // 浮点数占4个字节，即 float 类型
                    value = state.getValue().getObjectAsFloat();
                    break;
                case JIVariant.VT_R8: // 浮点数占8个字节，即 double 类型
                    value = state.getValue().getObjectAsDouble();
                    break;
                case JIVariant.VT_BSTR: // 字符串
                    value = state.getValue().getObjectAsString();
                    break;
                case JIVariant.VT_UI2: // 无符号占2个字节，即 word 类型，字
                case JIVariant.VT_UI4: // 无符号占4个字节，即 Dword 类型，双字
                    value = state.getValue().getObjectAsUnsigned().getValue();
                    break;
                case JIVariant.VT_DATE: // 日期 类型
                    value = state.getValue().getObjectAsDate();
                    break;
                case JIVariant.VT_BOOL: // 布尔类型
                    value = state.getValue().getObjectAsBoolean();
                    break;
                case JIVariant.VT_ARRAY | JIVariant.VT_I2: // short 数组 类型
                case JIVariant.VT_ARRAY | JIVariant.VT_R4: // float 数组 类型
                case JIVariant.VT_ARRAY | JIVariant.VT_BSTR: // 字符串 数组 类型
                case JIVariant.VT_ARRAY | JIVariant.VT_I8: // long 数组 类型
                case JIVariant.VT_ARRAY | JIVariant.VT_I4: // int 数组 类型
                    value = state.getValue().getObjectAsArray().getArrayInstance();
                    break;
                case JIVariant.VT_EMPTY:
                    break;
                default:
                    value = state.getValue().getObject();
                    break;
            }
            String itemId = item.getId();
            //String deviceName = itemId.substring(0, itemId.lastIndexOf("."));
            String fieldKey = itemMappings.get(itemId);
            SimpleDateFormat sdfStandard = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            data.put(JsonItemConst.g_device_id, deviceSn);
            data.put(JsonItemConst.g_generation_time, sdfStandard.format(state.getTimestamp().getTime()));
            data.put(JsonItemConst.g_collect_time, DateTimeUtil.getCurrentTime());
            data.put(fieldKey, value);
            String sJsonData = JsonUtil.toString(data);
            log.info("解析后的数据:"+ sJsonData);
            try {
                TransmitToMqtt.pushData(deviceSn, "admin", "admin", sJsonData);
            }catch (Exception e){
                e.printStackTrace();
            }
            //itemValue.put(item.getId(), value);
        } catch (JIException e) {
            throw new RuntimeException(e);
        }
    }
}
