package tofly.opc_gateway.transmit;

import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import tofly.opc_gateway.log.Log;
import tofly.opc_gateway.log.LogFactory;
import tofly.opc_gateway.util.SystemConst;

import java.util.concurrent.ConcurrentHashMap;

/**
 * 报文转发管理（MQTT）
 */
public class TransmitToMqtt {


    /**
     * 日志
     */
    private static final Log log = LogFactory.getLog(TransmitToMqtt.class);

    private static ConcurrentHashMap<String, MqttClient> mqttClientCache = new ConcurrentHashMap<>();

    public static void pushData(String p_sDeviceId, String p_sMqttUsername,
                                String p_sMqttPassword, String p_sContent) {
        log.info("设备ID：" + p_sDeviceId);
        mqttClientCache.computeIfAbsent(p_sDeviceId, k -> connect(p_sDeviceId, p_sMqttUsername, p_sMqttPassword));
        MqttClient mqttClient = mqttClientCache.get(p_sDeviceId);
        try {
            if (!mqttClient.isConnected()) {
                mqttClient.connect(getMqttConnectOptions(p_sMqttUsername, p_sMqttPassword));
            }
            MqttMessage mm = new MqttMessage(p_sContent.getBytes());
            int iQos = 2;
            mm.setQos(iQos);
            mqttClient.publish("v1/devices/me/telemetry", mm);
            log.info("Message published");
        } catch (MqttException e) {
            e.printStackTrace();
        }

    }

    private static MqttClient connect(String p_sDeviceId, String p_sMqttUsername, String p_sMqttPassword) {
        String sBroker = "tcp://" + SystemConst.g_platform_ip + ":" + SystemConst.g_mqtt_port;
        MemoryPersistence mp = new MemoryPersistence();
        try {
            MqttClient mcClient = new MqttClient(sBroker, p_sDeviceId, mp);
            mcClient.setCallback(new OnMessageCallback()); // 设置回调
            MqttConnectOptions mco = getMqttConnectOptions(p_sMqttUsername, p_sMqttPassword);

            try {
                log.info("连接MQTT服务: " + sBroker);
                mcClient.connect(mco);// 建立连接
            } catch (MqttException e) {
                throw new RuntimeException(e);
            }
            log.info("连接MQTT服务成功: " + sBroker);
            return mcClient;
        } catch (MqttException e) {
            throw new RuntimeException(e);
        }
    }

    private static MqttConnectOptions getMqttConnectOptions(String p_sMqttUsername, String p_sMqttPassword) {
        // MQTT 连接选项
        MqttConnectOptions mco = new MqttConnectOptions();
        mco.setUserName(p_sMqttUsername);
        mco.setPassword(p_sMqttPassword.toCharArray());
        mco.setAutomaticReconnect(true);//自动重连
        // 保留会话
        mco.setCleanSession(true);
        return mco;
    }

    public static void disconnect(String p_sDeviceId) {
        mqttClientCache.computeIfPresent(p_sDeviceId, (s, mqttClient) -> {
            try {
                mqttClient.disconnect();
                return mqttClient;
            } catch (MqttException e) {
                throw new RuntimeException(e);
            }
        });
    }


    public static class OnMessageCallback implements MqttCallback {
        public void connectionLost(Throwable p_t) {
            log.info("连接断开");
        }

        public void messageArrived(String p_sTopic, MqttMessage p_mm) throws Exception {
            log.info("接收消息主题:" + p_sTopic);
            log.info("接收消息Qos:" + p_mm.getQos());
            log.info("接收消息内容:" + new String(p_mm.getPayload()));
        }

        public void deliveryComplete(IMqttDeliveryToken p_mdtToken) {
            log.info("消息转发完成:" + p_mdtToken.isComplete());
        }
    }
}
